#ifndef __SYMBOL_H__
#define __SYMBOL_H__

/*
 * "Copyright (c) 2012 by Fritz Sieker."
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written
 * agreement is hereby granted, provided that the above copyright notice
 * and the following two paragraphs appear in all copies of this software,
 * that the files COPYING and NO_WARRANTY are included verbatim with
 * any distribution, and that the contents of the file README are included
 * verbatim as part of a file named README with any distribution.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE AUTHOR
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 */

/** @file symbol.h
 *  @brief Defines the interface to symbol.c functions (do not modify)
 *  @details This file defines the interface to a C file field.c that
 *  you will complete. The underlying data structure(s) used will be
 *  defined by the actual assignment and may be coordinated with work
 *  in the cs200 Data Structures course.
 *  <p>
 *  In this implementation, you will learn about conditional compilation
 *  and about dynamic memory management using malloc/free. You will also
 *  learn about function pointers (callback functions). It is not
 *  expected that this be a general data structure in the sense that multiple
 *  "instances" may be created. Rather it is a single data structure used
 *  in both the LC3 assemble and LC3 simulator. Some of the routines are
 *  used only in the simulator, hence the conditional compilation.
 */

/** The symbol_find methods return a pointer to this data structure. It is up
 *  to the implementor to decide how to use this stucture in the implementation.
 */

typedef struct symbol {
    char* name; /**< the name of the symbol */
    int   addr; /**< symbol's address in the LC3 memory */
} symbol_t;

/** initialize the symbol table code. Should only be called <b>once</b>.
 *  @param searchByAddr - if 0, then additonal data structures needed to
 *  find a symbol by is address are not created. If non-zero, they are created.
 */
void symbol_init (int searchByAddr);

/** Clear the symbol table. The must NOT be any memory leaks.
 */
void symbol_reset (void);

/** Add a symbol to the symbol table. If MAP_LOCATION_TO_SYMBOL is defined,
 *  information will be added to a 2nd data structure that gives access to
 *  symbols by address. See symbol_find_at_addr.
 *  @param name - the name of the symbol
 *  @param addr - the address of the symbol
 *  @return 1 if the symbol is not currently in the symbol table, 0 is the
 *   symbol already exists.
 */

int symbol_add (const char* name, int addr);

/** Find a symbol by its name
 *  @param name - the symbols name (case independent).
 *  @return  the symbols information or NULL if no symbol is associated with
 *   the name.
 */
symbol_t* symbol_find_by_name (const char* name);

/** Defines the signature of a callback function (also known as a function
 *  pointer). This is how languages such as Java and C++ do <b>dynamic</b>
 *  binding (i.e. figure out which function to call). Recall that in Java the
 *  code <code>obj.equals(object)</code> will call one of possibly many
 *  different methods depending on the actual type of <code>obj</code>. This
 *  is because the method <b>.equals()</b> may be <b>overridden</b>.
 *  <p>
 *  In the LC3 this code uses the <b>JSRR</b> opcode. With this opcode, the
 *  address of the routine to call is stored in a register and can be
 *  changed at runtime. Compare this to a <b>JSR nameOfRoutine</b> opcode which
 *  specifies what routine to call from the label that follows it. Thus, the
 *  address is fixed at <b>assembly</b> time. 
 *  <p>
 *  This is used in the symbol_iterate() function. An interesting variation
 *  would be to have the callback function return an integer which determines
 *  whether the iteration should contibue or terminate.
 */
typedef void (*iterate_fnc_t)(symbol_t* sym, void* data);

/** This function calls the function for every entry in the symbol table.
 *  The assigment will define the order in which the entries should be visited.
 *  @param fnc - the function to be called on every element
 *  @param data - any additional information to be passed on to fnc. The called
 *  function will cast this to whatever type was actually passed.
 */
void symbol_iterate (iterate_fnc_t fnc, void* data);

/** Find a symbol by its address. Return <code>null</code> if no symbol is
 *  associated with the address or the parameter to <code>symbol_init()</code>
 *  was zero. The lc3 assembler calls <code>lc3_init()</code> with zero, while
 *  the simulator calls it with a value of 1. Your assignment will define
 *  whether it is an error or not to have multiple symbols with the same
 *  address and if it must store multiple symbols for an address, or only the
 *  first symbol added that has that address. The function need not handle
 *  addresses beyond the size of memory of the LC3 memory.
 *  @param addr - the address
 *  @return a pointer to the symbol at this address or <code>null</code>
 */
symbol_t* symbol_find_at_addr (int addr);

/** Remove symbol(s) given an address. At completion, the symbol is removed from
 *  the symbol table and also removed from the data structure of names indexed
 *  by address Used by symbol_find_at_addr().. This function is ONLY availabe
 *  if the implemtation is compiled with <b>MAP_LOCATION_TO_SYMBOL</b> defined.
 *  This function is used in the simulator, but not in the assembler. 
 *  @param addr - remove symbol(s) associated with this address.
 */  
void symbol_remove_at_addr (int addr);

#endif /* __SYMBOL_H__ */

